/*
 * Linux Ethernet RSTP (IEEE 802.1w) bridging implementation
 * Copyright (C) 2003-2004 IMMS gGmbH, <http://www.imms.de/>
 *
 * Authors:
 * Marco Goetze, <marco.goetze@imms.de>
 *
 * This code builds upon parts of the original STP bridging implementation by
 * Lennert Buytenhek, <buytenh@gnu.org>.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#ifndef _BR_RSTP_STM_H
#define _BR_RSTP_STM_H

#include "br_private.h"

/* auxiliary declarations... */

#define BR_RSTP_STM_STATE_BEGIN 0
#define BR_RSTP_STM_TICK (HZ/CONFIG_BRIDGE_STM_STEPS)

#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

extern void br_rstp_stm_init_bridge_states(struct net_bridge*);
extern void br_rstp_stm_init_port_states(struct net_bridge_port*);

extern void br_rstp_stm_step(struct net_bridge*);

/* per-bridge STMs...*/

#define BR_RSTP_STMS_BRIDGE 1

typedef void (*br_rstp_stm_bridge_function)(struct net_bridge*, int*);

extern br_rstp_stm_bridge_function br_rstp_stm_port_role_selection;  /* 17.22 */

/* per-port STMs... */

#define BR_RSTP_STMS_PORTS 6

typedef void (*br_rstp_stm_port_function)(struct net_bridge_port*, int*);

extern br_rstp_stm_port_function br_rstp_stm_port_information;         /* 17.21 */
extern br_rstp_stm_port_function br_rstp_stm_port_protocol_migration;  /* 17.26 */
extern br_rstp_stm_port_function br_rstp_stm_port_role_transitions;    /* 17.23 */
extern br_rstp_stm_port_function br_rstp_stm_port_transmit;            /* 17.27 */
extern br_rstp_stm_port_function br_rstp_stm_port_state_transition;    /* 17.24 */
extern br_rstp_stm_port_function br_rstp_stm_topology_change;          /* 17.25 */

/*
 * NOTE: 17.20, the Port Timers state machine, is implemented implicitly using
 *       a custom representation of timers which eliminates the need to
 *       decrement timers periodically.
 */

/*
 * NOTE: 18.2 (IEEE 802.1t), the Bridge Detection STM, is handled implicitly
 *       by the input code.
 */

#endif  /* _BR_RSTP_STM_H */
